const connect = require("usp-js").default;
const fs = require("fs");
const crypto = require("crypto");

const convertBase64 = (path) => {
  const bitmap = fs.readFileSync(path);
  return bitmap.toString('base64');
};

const md5sum = (path) => {
  const content = fs.readFileSync(path);
  return crypto.createHash("md5").update(content).digest("hex");
};

function splitStringBySegmentLength(source, segmentLength) {
  const target = [];
  if (!segmentLength || segmentLength < 1)
    return target;

  for (
      const array = Array.from(source);
      array.length;
      target.push(array.splice(0,segmentLength).join('')));
  return target;
}

const run = async () => {
  const filePath = "local_file_full_path"; // TODO change this to the path of the local
  					   // file that to be downloaded

  const data = convertBase64(filePath);
  const md5 = md5sum(filePath);
  const data_len = data.length;
  const {size} = fs.statSync(filePath);

  console.log("Encoded Data length:" + data_len);
  console.log("Download file size:" + size);

  /* Now lets send 1MB fragments each time */
  const n_bytes = 1 * 1024 * 1024;

  fragments = splitStringBySegmentLength(data, n_bytes);
  const total_frag = fragments.length;
  if (total_frag == 0) {
    console.log("Failed to create fragments");
    return;
  }

  // Connect
  const usp = await connect({
    host: "192.168.1.1",
    username: "admin",
    password: "admin",
    port: 9001,
    protocol: "ws",
  });

  var sequence = 0;
  var result = true;
  var command_key = crypto.randomInt(111111, 999999); /* (MIN, MAX) */

  for (var frag of fragments) {
    sequence++;

    const [download] = await usp.operate("Device.LocalAgent.X_IOPSYS_EU_Download()", {
      isAsync: false,
    });

    values = {
      Filename: "image",
      Content: `${frag}`,
      Filesize: `${size}`,
      Checksum: `${md5}`,
      SequenceNumber: `${sequence}`,
      TotalFragment: `${total_frag}`,
      CommandKey: `${command_key}`
    };

    const result1 = await download(values);

    if (result1 == null) {
      console.log("No response received");
      result = false;
      break;
    } else {
      console.log("Download test result:");
      console.log(result1);

      if (result1.outputArgs.Result == "Failed") {
        result = false;
        break;
      }
    }
  }

  if (result == true)
    console.log("Operation succeed");
  else
    console.log("Operation failed");

  // Disconnect
  await usp.disconnect();
};

run();

